@@ -128,6 +128,8 @@ static initializer cpu_flag_init[] =
     "CpuMMX|CpuSSE|CpuSSE2|CpuSSE3|CpuSSSE3|CpuSSE4_1|CpuSSE4_2|CpuAVX|CpuFMA" },
   { "CPU_FMA4_FLAGS",
     "CpuMMX|CpuSSE|CpuSSE2|CpuSSE3|CpuSSSE3|CpuSSE4_1|CpuSSE4_2|CpuAVX|CpuFMA4" },
+  { "CPU_XOP_FLAGS",
+    "CpuMMX|CpuSSE|CpuSSE2|CpuSSE3|CpuSSSE3|CpuSSE4_1|CpuSSE4_2|CpuSSE4a|CpuABM|CpuAVX|CpuFMA4|CpuXOP" },
   { "CPU_LWP_FLAGS",
     "CpuLWP" },
   { "CPU_MOVBE_FLAGS",
@@ -298,6 +300,7 @@ static bitfield cpu_flags[] =
   BITFIELD (CpuPCLMUL),
   BITFIELD (CpuFMA),
   BITFIELD (CpuFMA4),
+  BITFIELD (CpuXOP),
   BITFIELD (CpuLWP),
   BITFIELD (CpuLM),
   BITFIELD (CpuMovbe),
@@ -359,9 +362,11 @@ static bitfield opcode_modifiers[] =
   BITFIELD (Vex0F),
   BITFIELD (Vex0F38),
   BITFIELD (Vex0F3A),
+  BITFIELD (XOP08),
   BITFIELD (XOP09),
   BITFIELD (XOP0A),
   BITFIELD (Vex3Sources),
+  BITFIELD (Vex2Sources),
   BITFIELD (VexImmExt),
   BITFIELD (SSE2AVX),
   BITFIELD (NoAVX),
